/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

/**
 * The bricks that appear in each level.
 * @author Naomi (boyfailure.dev)
 * @since 20250323
 */
public class Brick extends Entity {

    private boolean isBroken = false;
    byte brickColor = 0;
    byte scoreOnBreak = 10;
    byte specialAbility = 0;
    Hitbox hitBoxLeft;
    Hitbox hitBoxRight;

    private GameState gameState;

    /**
     * Constructs a brick.
     * @param gameState The GameState in which the Brick will reside.
     * @param x The x coordinate of the Brick.
     * @param y The y coordinate of the Brick.
     * @param isBroken Determines whether the Brick is broken.
     * @param brickColor The color code of the Brick.
     * @param score The score the Player earns upon breaking the Brick.
     * @param specialAbility A special event that occurs upon breaking the Brick.
     */
    public Brick(GameState gameState, int x, int y, boolean isBroken, int brickColor, int score, int specialAbility) {
        this.width = 40;
        this.height = 17;
        this.gameState = gameState;
        this.posX = (short) x;
        this.posY = (short) y;
        this.isBroken = isBroken;
        this.brickColor = (byte) brickColor;
        this.scoreOnBreak = (byte) score;
        this.hitBox = new Hitbox(x + 2, y, this.width - 4, this.height, 0);
        this.hitBoxLeft = new Hitbox(x, y + 2, 6, this.height - 4, 0);
        this.hitBoxRight = new Hitbox(x + this.width - 6, y + 2, 6, this.height - 4, 0);
        this.specialAbility = (byte) specialAbility;
    }

    public void breakBrick() {
        if (!isBroken) {
            isBroken = true;
            gameState.addScore(this.scoreOnBreak + (gameState.getCurrentLevelID() - 1));
            gameState.getTextElements().get(3).setText(String.valueOf(gameState.getScore()));
            switch(this.specialAbility) {
                case 1:
                    gameState.addLife();
                    break;
                case 2:
                    while (gameState.getCollectorActiveState()) {}
                    Ball newBall = new Ball(gameState, 300, 300, 7, 7, 0, true);
                    this.gameState.getBalls().add(newBall);
                    newBall.spawnBall((this.posX + (this.width / 2)), (this.posY + (this.height / 2)));
                    break;
                default:
                    break;
            }

            // Show score on break
            TextElement bruh = new TextElement(this.gameState,
                                                String.valueOf(this.scoreOnBreak),
                                                0.25f,
                                                this.posX + (this.width / 3),
                                                this.posY - 3,
                                                this.brickColor,
                                                1,
                                                300);
            bruh.activateTimer();
            bruh.setActiveState(true);
            //bruh.moveUpOnFrame();
            this.gameState.getTextElements().add(bruh);
        }

        // Draw particles
        Particle particle1, particle2, particle3;
        particle1 = new Particle(gameState, 0, this.posX + 12, this.posY + 3, 1, 1, this.brickColor, 2, 4, 1);
        particle2 = new Particle(gameState, 0, this.posX + 20, this.posY + 13, 1, 1, this.brickColor, 2, 3, 1);
        particle3 = new Particle(gameState, 0, this.posX + 32, this.posY + 7, 1, 1, this.brickColor, 2, 2, 1);
        gameState.getParticles().add(particle1);
        gameState.getParticles().add(particle2);
        gameState.getParticles().add(particle3);
        particle1.spawn(5);
        particle2.spawn(5);
        particle3.spawn(5);

        this.posX = 32767;
        this.posY = 32767;
        this.hitBox.moveTo(32760, 32760, 2, 2);
        this.hitBoxLeft.moveTo(32760, 32760, 2, 2);
        this.hitBoxRight.moveTo(32760, 32760, 2, 2);
    }

    public boolean getBrokenState() {
        return this.isBroken;
    }
    public void setBrokenState(boolean brokenState) {
        this.isBroken = brokenState;
    }

}
