/*
 * Copyright 2025 naomi.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.awt.Point;

/**
 * Base class for all entities in the game.
 * @author Naomi (boyfailure.dev)
 * @since 1.0
 */
public class Entity {

    /** The horizontal position of the entity, in pixels. */
    short posX;
    /** The vertical position of the entity, in pixels. */
    short posY;
    /** The width of the entity, in pixels. */
    short width;
    /** The height of the entity, in pixels. */
    short height;
    /** The default hitbox of the entity. */
    Hitbox hitBox;

    //<editor-fold defaultstate="collapsed" desc=" Accessor methods (get/set) ">

    /**
     * Returns the position of the entity as a Point.
     * @return the position of the entity
     */
    public Point getPosition() {
        return new Point(this.posX, this.posY);
    }
    /**
     * Returns the horizontal position of the entity.
     * @return the horizontal position of the entity
     */
    public short getPositionX() {
        return this.posX;
    }
    /**
     * Returns the vertical position of the entity.
     * @return the vertical position of the entity
     */
    public short getPositionY() {
        return this.posY;
    }

    //</editor-fold>

}
