/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.awt.Color;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import javax.swing.JFrame;

/**
 * The window in which the game is displayed.
 * @author Naomi (boyfailure.dev)
 * @since 1.0
 */
public class GameFrame extends JFrame {

    private GameState gameState;
    private GameDisplay gameDisplay;

    public GameFrame(GameState gameState, GameDisplay gameDisplay) {
        this.gameState = gameState;
        this.gameDisplay = gameDisplay;

        // Set the game window's properties
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        this.setTitle(this.gameState.getGameName());
        this.setSize(this.gameState.getWindowResX(), this.gameState.getWindowResY());
        this.getContentPane().setBackground(Color.black);
        this.getContentPane().add(this.gameDisplay);

        // Add the component adapter to change vector drawing scale when the window is resized
        this.addComponentListener(this.getComponentAdapter());

        // Add the key listener to get player input
        this.addKeyListener(this.gameState.getKeyListener());
    }

    //<editor-fold defaultstate="collapsed" desc=" Event listeners ">

    public ComponentAdapter getComponentAdapter() {
        return new GameComponentAdapter();
    }
    class GameComponentAdapter extends ComponentAdapter {
        public void componentResized(ComponentEvent componentEvent) {
            GameFrame.this.gameState.setWindowResX((short) GameFrame.this.getBounds().width);
            GameFrame.this.gameState.setWindowResY((short) GameFrame.this.getBounds().height);
            GameFrame.this.gameDisplay.setBeamScaleX(GameFrame.this.gameState.getWindowResX() / 800f);
            GameFrame.this.gameDisplay.setBeamScaleY(GameFrame.this.gameState.getWindowResY() / 600f);
            if (GameFrame.this.gameDisplay.getBeamScaleX() <= GameFrame.this.gameDisplay.getBeamScaleY()) {GameFrame.this.gameDisplay.setBeamThicknessScale(GameFrame.this.gameDisplay.getBeamScaleX());}
            else {GameFrame.this.gameDisplay.setBeamThicknessScale(GameFrame.this.gameDisplay.getBeamScaleY());}
        }
    }

    //</editor-fold>

}
