/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.awt.geom.Rectangle2D;

/**
 * Invisible regions in each object that dictate when it collides with other objects.
 * @author Naomi (boyfailure.dev)
 * @since 20250327
 */
public class Hitbox {
    
    private Rectangle2D bounds;
    byte padding = 0;
    
    public Hitbox(int x, int y, int x2, int y2, int padding) {
        this.padding = (byte) padding;
        this.bounds = new Rectangle2D.Float(x - padding, y - padding, x2 + padding, y2 + padding);
    }
    
    public boolean collides(Rectangle2D hitBox2) {
        if (this.bounds.intersects(hitBox2)) {
            return true;
        }
        else {
            return false;
        }
    }
    
    public void moveTo(int x, int y, int x2, int y2) {
        this.bounds.setRect(x - this.padding, y - this.padding, x2 + padding, y2 + padding);
    }
    
    public Rectangle2D getBounds() {
        return this.bounds;
    }
    public void setBounds(int x, int y, int x2, int y2) {
        this.bounds = new Rectangle2D.Float(x - this.padding, y - this.padding, x2 + this.padding, y2 + this.padding);
    }

}
