/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.util.ArrayList;

/**
 * The progressive levels that contain different brick layouts.
 * @author Naomi (boyfailure.dev)
 * @since 20250325
 */
public class Level {
    
    short levelID;
    byte rowHeight = 21;
    String levelName = "LEVEL";
    byte[] levelLayout; // 19 bricks per row
    byte[] colorMap;

    ArrayList<Brick> brickList;

    private GameState gameState;

    public Level(GameState gameState, int id) {
        this.gameState = gameState;
        this.levelID = (short) id;

        switch(this.levelID) {
            case 1:
                this.levelName = "HELLO WORLD";
                this.levelLayout = new byte[]
                {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                 3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,
                 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
                 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
                this.colorMap = new byte[]
                {10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12};
                break;
            case 2:
                this.levelName = "DOUBLE TROUBLE";
                this.levelLayout = new byte[]
                {4,4,4,4,4, 4,4,4,4,4,4,4,4, 4,4,4,4,4,4,
                 3,3,3,3,3,16,3,3,3,3,3,3,3,16,3,3,3,3,3,
                 2,2,2,2,2, 2,2,2,2,2,2,2,2, 2,2,2,2,2,2,
                 1,1,1,1,1, 1,1,1,1,1,1,1,1, 1,1,1,1,1,1};
                this.colorMap = new byte[]
                {10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,18,13,13,13,13,13,13,13,18,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12};
                break;
            case 3:
                this.levelName = "THE GAP";
                this.levelLayout = new byte[]
                {5,5,5,5,5, 5,5,5,0,0,0,5,5, 5,5,5,5,5,5,
                 4,4,4,4,4, 4,4,4,0,0,0,4,4, 4,4,4,4,4,4,
                 3,3,3,3,3,16,3,3,0,0,0,3,3,16,3,3,3,3,3,
                 2,2,2,2,2, 2,2,2,0,0,0,2,2, 2,2,2,2,2,2,
                 1,1,1,1,1, 1,1,1,0,0,0,1,1, 1,1,1,1,1,1};
                this.colorMap = new byte[]
                {14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
                 10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,18,13,13,13,13,13,13,13,18,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12};
                break;
            case 4:
                this.levelName = "HOLE IN THE WALL";
                this.levelLayout = new byte[]
                {4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                 3,3,0,0,3,3,3,3,0,0,0,3,3,3,3,0,0,3,3,
                 2,2,0,0,2,2,2,2,0,0,0,2,2,2,2,0,0,2,2,
                 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
                this.colorMap = new byte[]
                {10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12};
                break;
            case 5:
                this.levelName = "A SECOND OPINION";
                this.levelLayout = new byte[]
                {0,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,0,
                 0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,
                 0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,
                 0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,
                 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
                 0,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,0,
                 0,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,3,0,
                 0,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,0,
                 0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0};
                this.colorMap = new byte[]
                {10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
                  0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
                 10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12};
                break;
            case 6:
                this.levelName = "SLICES";
                this.levelLayout = new byte[]
                {5,5,0,5,5,0,5,5,0,5,5,0,5,5,0,5,5,0,5,
                 4,0,4,4,0,4,4,0,4,4,0,4,4,0,4,4,0,4,4,
                 0,3,3,0,3,3,0,3,3,0,3,3,0,3,3,0,3,3,0,
                 2,2,0,2,2,0,2,2,0,2,2,0,2,2,0,2,2,0,2,
                 1,0,1,1,0,1,1,0,1,1,0,1,1,0,1,1,0,1,1,
                 0,4,4,0,4,4,0,4,4,0,4,4,0,4,4,0,4,4,0,
                 3,3,0,3,3,0,3,3,0,3,3,0,3,3,0,3,3,0,3,
                 2,0,2,2,0,2,2,0,2,2,0,2,2,0,2,2,0,2,2,
                 0,1,1,0,1,1,0,1,1,0,1,1,0,1,1,0,1,1,0};
                this.colorMap = new byte[]
                {14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,14,
                 10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,
                 10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,10,
                 13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,13,
                  8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8, 8,
                 12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12,12};
                break;
            case 1001:
                this.levelName = "TRANS RIGHTS";
                this.levelLayout = new byte[]
                {5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,5,
                 4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,
                 3,3,3,3,3,3,3,3,3,3,3,3,3,5,3,3,3,3,3,
                 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
                 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
                this.colorMap = new byte[]
                {20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,
                 19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
                  1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
                 19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,19,
                 20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20,20};
                break;
            case 1002:
                this.levelName = "I USE ARCH BTW";
                this.levelLayout = new byte[]
                {0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,6,6,6,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,0,5,5,5,0,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,0,4,4,0,4,4,0,0,0,0,0,0,0,
                 0,0,0,0,0,0,3,3,0,0,0,3,3,0,0,0,0,0,0,
                 0,0,0,0,0,2,2,2,0,0,0,2,2,2,0,0,0,0,0,
                 0,0,0,1,1,1,1,1,0,0,0,1,1,1,1,1,0,0,0};
                this.colorMap = new byte[]
                {16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,
                 16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16,16};
                break;
            case 32767:
                this.levelName = "DEBUG.LEVEL;";
                this.levelLayout = new byte[]
                {1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,
                 2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,
                 3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,
                 4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,
                 5,6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,
                 6,7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,
                 7,1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,
                 1,2,3,4,5,6,7,1,2,3,4,5,6,7,1,2,3,4,5};
                this.colorMap = new byte[]
                { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,
                  1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18, 0,
                  2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18, 0, 1,
                  3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18, 0, 1, 2,
                  4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18, 0, 1, 2, 3,
                  5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18, 0, 1, 2, 3, 4,
                  6, 7, 8, 9,10,11,12,13,14,15,16,17,18, 0, 1, 2, 3, 4, 5,
                  7, 8, 9,10,11,12,13,14,15,16,17,18, 0, 1, 2, 3, 4, 5, 6};
                break;
            default:
                this.levelName = "LEVEL 404: NOT FOUND (" + this.levelID + ")";
                this.levelLayout = new byte[]
                {1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
                 1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1};
                this.colorMap = new byte[]
                {2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
                 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
                 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,
                 2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2};
                break;
        }

    }

    public void constructLevel() {
        this.brickList = new ArrayList<>();
        // Brick placement
        short brickLeftPlacement = 20;
        for (int i = 0; i < this.levelLayout.length; i++) {
            switch (this.levelLayout[i]) {
                case 1:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 4, 0));
                    break;
                case 2:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 8, 0));
                    break;
                case 3:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 16, 0));
                    break;
                case 4:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 32, 0));
                    break;
                case 5:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 64, 0));
                    break;
                case 6:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 8, 0));
                    break;
                case 7:
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 8, 0));
                    break;
                case 16: // Spawns a ball
                    this.brickList.add(new Brick(gameState, brickLeftPlacement, (((i / 19)) * this.rowHeight) + 50, false, this.colorMap[i], 8, 2));
                    break;
                default:
                    break;
            }
            brickLeftPlacement += 40;
            if (brickLeftPlacement >= 760) {brickLeftPlacement = 20;}
        }
        this.startLevel();
    }

    public void startLevel() {
        gameState.getTextElements().set(7, new TextElement(gameState, this.levelName, 1, 24, 480, 15, 2, 3000));
        gameState.getTextElements().get(7).activateTimer();
    }
    
}