/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.util.ArrayList;

/**
 * Customizable game menus.
 * @author Naomi (boyfailure.dev)
 * @since 1.0
 */
public class Menu {
    
    public String menuTitle = "Menu";
    public byte index = 0;

    private ArrayList<TextElement> textElementList;
    
    private GameState gameState;
    
    public Menu(GameState gameState) {
        this.gameState = gameState;
    }
    
    /**
     * Adds a TextElement to the Menu.
     * @param text The text that will be displayed.
     * @param textScale The size of the text on screen.
     * @param x The x coordinate of the text's origin point.
     * @param y The y coordinate of the text's origin point.
     * @param textColor The color of the text.
     * @param vectorScale The scale of the text.
     */
    public void addTextElement(String text, float textScale, int x, int y, int textColor, float vectorScale) {
        textElementList.add(new TextElement(this.gameState, text, textScale, x, y, textColor, vectorScale));
    }
    
    /**
     * Gets the TextElement at the specified index in the Menu.
     * @param index
     * @return The TextElement at the index.
     */
    public TextElement getTextElement(int index) {
        return textElementList.get(index);
    }
    
    public void activate() {
        
    }
    
}
