/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

/**
 * The player-controlled paddle that guides balls into bricks.
 * @author Naomi (boyfailure.dev)
 * @since 20250323
 */
public class Paddle extends Entity {
    
    byte paddleColor;
    byte paddleSpeed = 0;
    boolean isActive = true;
    float speedMultiplier = 1;
    Hitbox hitBoxLeft;
    Hitbox hitBoxRight;
    
    private GameState gameState;
    
    public Paddle(GameState gameState, int x, int y, int width, int height, int paddleColor) {
        this.gameState = gameState;
        this.posX = (short) x;
        this.posY = (short) y;
        this.width = (short) width;
        this.height = (short) height;
        this.paddleColor = (byte) paddleColor;
        this.hitBox = new Hitbox(x, y, width, height, 4);
        this.hitBoxLeft = new Hitbox(x, y, width / 5, height, 4);
        this.hitBoxRight = new Hitbox(x + ((width / 5) * 4), y, width / 5, height, 4);
    }
    
    public void movePaddle(boolean direction) {
        if (this.paddleSpeed <= 15) {this.paddleSpeed += 3;}
        if (direction) {this.posX += (this.paddleSpeed * this.speedMultiplier);}
        else {this.posX -= (this.paddleSpeed * this.speedMultiplier);}
        if (this.posX <= 20) {this.posX = 20;}
        else if (this.posX >= (gameState.getInternalResX() - this.width - 20)) {this.posX = (short) (gameState.getInternalResX() - this.width - 20);}
        
        this.hitBox.moveTo(this.posX, this.posY, this.width, this.height);
        this.hitBoxLeft.moveTo(this.posX, this.posY, this.width / 5, this.height);
        this.hitBoxRight.moveTo(this.posX + ((this.width / 5) * 4), this.posY, this.width / 5, this.height);
    }
    
    public void cullPaddle() {
        isActive = false;
        this.posX = 32767;
        this.posY = 32767;
        this.hitBox.setBounds(32766, 32766, 32767, 32767);
        this.hitBoxLeft.setBounds(32766, 32766, 32767, 32767);
        this.hitBoxRight.setBounds(32766, 32766, 32767, 32767);
    }

    public boolean getActiveState() {
        return this.isActive;
    }
    public void setActiveState(boolean activeState) {
        this.isActive = activeState;
    }

}
