/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

/**
 * Small graphical effects that appear on screen.
 * @author Naomi (boyfailure.dev)
 * @since 20250325
 */
public class Particle {
    
    byte particleID;
    short particleX;
    short particleY;
    short particleX2;
    short particleY2;
    boolean isActive = true;
    byte particleColor;
    short particleFrame = 0;
    short particleLifetime;
    float vectorScale = 1;
    byte particleDirection;
    byte particleSpeed = 2;
    
    private GameState gameState;
    
    public Particle(GameState gameState, int particleID, int x, int y, int x2, int y2, int particleColor, float vectorScale, int direction, int speed) {
        this.gameState = gameState;
        this.particleID = (byte) particleID;
        this.particleX = (short) x;
        this.particleY = (short) y;
        this.particleX2 = (short) x2;
        this.particleY2 = (short) y2;
        this.particleColor = (byte) particleColor;
        this.vectorScale = vectorScale;
        this.particleDirection = (byte) direction;
        this.particleSpeed = (byte) speed;
    }
    
    public void spawn(int lifetime) {
        this.isActive = true;
        // if (gameState.getConfettiMode()) {this.particleLifetime = 400;}
        // else {this.particleLifetime = (short) lifetime;}
        this.particleLifetime = (short) lifetime;
    }
    
    public void update() {
        this.particleFrame++;
        if (this.particleDirection != 0) {
            switch(this.particleDirection) {
                case 1:this.particleX += this.particleSpeed;break; // Right
                case 2:this.particleX += this.particleSpeed;this.particleY+= this.particleSpeed;break; // Down right
                case 3:this.particleY += this.particleSpeed;break; // Down
                case 4:this.particleX -= this.particleSpeed;this.particleY+= this.particleSpeed;break; // Down left
                case 5:this.particleX -= this.particleSpeed;break; // Left
                case 6:this.particleX -= this.particleSpeed;this.particleY-= this.particleSpeed;break; // Up left
                case 7:this.particleY -= this.particleSpeed;break; // Up
                case 8:this.particleX += this.particleSpeed;this.particleY-= this.particleSpeed;break; // Up right
            }
        }
        if (this.particleFrame >= this.particleLifetime) {
            this.isActive = false;
        }
    }

    public boolean getActiveState() {
        return this.isActive;
    }
    
}
