/*
 * Copyright 2025 Naomi (boyfailure.dev).
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package dev.boyfailure.vectorbreakout;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import javax.swing.Timer;

/**
 * The initial code to launch the game.
 * @author Naomi (boyfailure.dev)
 * @since 20250323
 */
public class VectorBreakout {

    public static GameState gameState;
    public static GameDisplay gameDisplay;
    public static GameFrame gameFrame;

    /**
     * This empty constructor is currently only in here because the main class
     * constructor generated in new projects on NetBeans 4. Will investigate to
     * see if this is necessary in JDK 1.5 next time I am using those tools.
     */
    public VectorBreakout() {}

    public static void main(String[] args) {

        // Enable OpenGL for hardware acceleration
        System.setProperty("sun.java2d.opengl", "true");

        // Initialize the components
        gameState = new GameState();
        gameDisplay = new GameDisplay(gameState);
        gameFrame = new GameFrame(gameState, gameDisplay);

        Timer gameTick = new Timer((1000 / gameState.getGameTickRate()), gameState.getGameTickActionListener());
        gameTick.start();

        Timer frameDisplay = new Timer((1000 / gameState.getTargetFrameRate()), new ActionListener() {
            public void actionPerformed(ActionEvent e) {
                gameState.incrementFrameCounter();
                gameFrame.repaint();
            }
        });
        frameDisplay.start();

        Timer gameCuller = new Timer(1000, gameState.getGameStateUpdateActionListener());
        gameCuller.start();

        // Show the game window
        gameState.showTitleScreen();
        gameFrame.setVisible(true);

    }
    
}
